function [ids_core_sources,ids_core_sources_description,varargout] = ...
  tcv_get_ids_core_sources(shot,ids_equil_empty,gdat_params,varargin)
%
% [ids_core_sources,ids_core_sources_description,varargout] = ...
%     tcv_get_ids_core_sources(shot,ids_equil_empty,gdat_params,varargin);
%
%
% gdat_params: gdat_data.gdat_params to get all params passed from original call
%

machine = 'tcv';
tens_time = -1;
tens_rho = -0.1;

if exist('gdat_params','var')
  [ids_core_sources, params_core_sources] = ...
    tcv_ids_headpart(shot,ids_equil_empty,'cores_sources','gdat_params',gdat_params,varargin{:});
else
  [ids_core_sources, params_core_sources] = ...
    tcv_ids_headpart(shot,ids_equil_empty,'cores_sources',varargin{:});
  aa=gdat_tcv;
  gdat_params = aa.gdat_params; % to get default params
end
params_eff_ref = gdat_params; params_eff_ref.doplot=0;
try params_eff_ref=rmfield(params_eff_ref,'source');catch;end % make sure no source (from ids def)

% initialize description
ids_core_sources_description = [];

%%
% save source template to fill for differnet source options
source_template = ids_core_sources.source{1};
profiles_template = source_template.profiles_1d{1};
globals_template = source_template.global_quantities{1};
%% get time grid and data from gdat 
ohm_gdat = gdat(shot,'ohm_data');
powers_gdat = gdat(shot,'powers');
% fix time grid to ohm time grid, same as bs

last_index = 0;
%% initialize source from template
% ohm
ohm_t_grid = powers_gdat.ohm.t; ohm_n_t = numel(ohm_t_grid);

id_ohm.description = 'Source from ohmic heating';
id_ohm.index = 7; id_ohm.name = 'ohmic';
ids_core_sources.source{last_index+1} = source_template;
ids_core_sources.source{last_index+1}.identifier = id_ohm;
ids_core_sources.source{last_index+1}.profiles_1d(1:ohm_n_t) = {profiles_template};
ids_core_sources.source{last_index+1}.global_quantities(1:ohm_n_t) = {globals_template};
ohm_data = ohm_gdat.ohm.ohm_data;

%fill profiles for times n t_grid

for ii = 1:ohm_n_t
  % ohm
  % 1d_profiles
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.time = ohm_t_grid(ii);
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.j_parallel = ...
    ohm_data.cd_dens.data(:,ii)';
  % globals
  ids_core_sources.source{last_index+1}.global_quantities{ii}.time = ohm_t_grid(ii);
  ids_core_sources.source{last_index+1}.global_quantities{ii}.power = powers_gdat.ohm.data(ii);
end

last_index = last_index+1;  % add if statement to only increment if ohmic source has been added

%% bs
bs_gdat = gdat(shot,'bs_data'); bs_data = bs_gdat.bs.bs_data;
bs_t_grid = bs_gdat.bs.t; bs_n_t = numel(bs_t_grid);

id_bs.description = 'Bootstrap current';
id_bs.index = 13; id_bs.name = 'bootstrap_current';
ids_core_sources.source{last_index+1} = source_template;
ids_core_sources.source{last_index+1}.identifier = id_bs;
ids_core_sources.source{last_index+1}.profiles_1d(1:ohm_n_t) = {profiles_template};
ids_core_sources.source{last_index+1}.global_quantities(1:ohm_n_t) = {globals_template};


% fill profiles for times n t_grid
for ii = 1:bs_n_t
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.time = bs_t_grid(ii);
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.j_parallel = ...
    bs_data.cd_dens.data(:,ii)'; 
end
 
last_index = last_index+1;  % add if statement to only increment if bs source has been added

%% ec
ec_gdat = gdat(shot,'ec_data');

if ~isempty(ec_gdat.ec.data) % if EC data available, fill sources
  
  ec_data = ec_gdat.ec.ec_data; ec_inputs = ec_gdat.ec.ec_inputs;
  % get tgrid from gdat ec_data
  ec_data_tgrid = ec_data.p_dens.t; nt_ec_data = numel(ec_data_tgrid);

  % get tgrid from gdat powers
  ec_powers_tgrid = powers_gdat.ec.t; nt_ec_powers = numel(ec_powers_tgrid);

  % retrieve active launcher information from ec_inputs
  nb_launchers = numel(ec_inputs.launchers_active.data);
  active_launchers = find(ec_inputs.launchers_active.data==1)';

  % Setup structures for active launchers from template
  main_desc = 'Source from electron cyclotron heating and current drive';
  id_ec.index = 3; id_ec.name = 'ec';
  for i_lau = active_launchers
    id_ec.description = sprintf('L%i/G%i, %s',i_lau,ec_inputs.gyro2launcher.data(i_lau),main_desc);
    ids_core_sources.source{last_index+i_lau} = source_template;
    ids_core_sources.source{last_index+i_lau}.identifier = id_ec;
    ids_core_sources.source{last_index+i_lau}.profiles_1d(1:nt_ec_powers) = {profiles_template};
    ids_core_sources.source{last_index+i_lau}.global_quantities(1:nt_ec_powers) = {globals_template};
  end

  % get data for globals from gdat powers and fill in ids_structure
  ec_total_pow = transpose(powers_gdat.ec.data(:,nb_launchers+1)); %use power from powers_gdat(injected ec power) instead of ec_data power
  ec_total_pow(isnan(ec_total_pow)) = 0;
  ec_total_cur = ec_data.cd_tot.data(nb_launchers+1,:);
  ec_total_cur(isnan(ec_total_cur)) = 0;

  for i_lau = active_launchers
    for ii = 1:nt_ec_powers
      ids_core_sources.source{last_index+i_lau}.global_quantities{ii}.time = ec_powers_tgrid(ii);
      ids_core_sources.source{last_index+i_lau}.global_quantities{ii}.power = ec_total_pow(ii);
      ids_core_sources.source{last_index+i_lau}.global_quantities{ii}.current_parallel = ec_total_cur(ii);
    end
  end

  % interpoating p_dens profiles from 'ec_data_tgrid' grid (toray tgrid) to 'ec_powers_tgrid' (powers tgrid)
  p_dens        = ec_data.p_dens.data;
  p_integrated  = ec_data.p_integrated.data;
  cd_dens       = ec_data.cd_dens.data;
  cd_integrated = ec_data.cd_integrated.data;

  p_ec_injected = powers_gdat.ec.data;
  ij = iround_os(ec_powers_tgrid,ec_data_tgrid);
  sparse_p_ec_injected = p_ec_injected(ij,:); % injected ec power vals corresponding to ec_data_tgrid
  n_rho = size(p_dens, 1);

  % calculate normalised profiles on ec_data_tgrid grid
  norm_p_dens        = zeros(n_rho,nb_launchers+1,nt_ec_data);
  norm_p_integrated  = zeros(n_rho,nb_launchers+1,nt_ec_data);
  norm_cd_dens       = zeros(n_rho,nb_launchers+1,nt_ec_data);
  norm_cd_integrated = zeros(n_rho,nb_launchers+1,nt_ec_data);
  for it = 1:nt_ec_data 
    norm_p_dens(:,:,it)        = p_dens(:,:,it)./sparse_p_ec_injected(it,:);
    norm_p_integrated(:,:,it)  = p_integrated(:,:,it)./sparse_p_ec_injected(it,:);
    norm_cd_dens(:,:,it)       = cd_dens(:,:,it)./sparse_p_ec_injected(it,:);
    norm_cd_integrated(:,:,it) = cd_integrated(:,:,it)./sparse_p_ec_injected(it,:); 
  end

  % interpolate normalised p_dens profiles on ec_powers_tgrid 
  interp_norm_p_dens        = zeros(n_rho,nb_launchers+1,nt_ec_powers);
  interp_norm_p_integrated  = zeros(n_rho,nb_launchers+1,nt_ec_powers);
  interp_norm_cd_dens       = zeros(n_rho,nb_launchers+1,nt_ec_powers);
  interp_norm_cd_integrated = zeros(n_rho,nb_launchers+1,nt_ec_powers);
  for i_lau = active_launchers
    for irho = 1:n_rho  
      % get power and current density at each rho
      trace_p_dens        = squeeze(norm_p_dens(irho,i_lau,:));
      trace_p_integrated  = squeeze(norm_p_integrated(irho,i_lau,:));
      trace_cd_dens       = squeeze(norm_cd_dens(irho,i_lau,:));
      trace_cd_integrated = squeeze(norm_cd_integrated(irho,i_lau,:));
      % interpolate on gdat powers tgrid
      interp_norm_p_dens(irho,i_lau,:)        = interp1(ec_data_tgrid,trace_p_dens,ec_powers_tgrid);
      interp_norm_p_integrated(irho,i_lau,:)  = interp1(ec_data_tgrid,trace_p_integrated,ec_powers_tgrid);
      interp_norm_cd_dens(irho,i_lau,:)       = interp1(ec_data_tgrid,trace_cd_dens,ec_powers_tgrid);
      interp_norm_cd_integrated(irho,i_lau,:) = interp1(ec_data_tgrid,trace_cd_integrated,ec_powers_tgrid);
    end
  end

  % normalised & interpolated profiles * p_ec_injected on ec_powers_tgrid
  interp_p_dens        = zeros(n_rho,nb_launchers+1,nt_ec_powers);
  interp_p_integrated  = zeros(n_rho,nb_launchers+1,nt_ec_powers);
  interp_cd_dens       = zeros(n_rho,nb_launchers+1,nt_ec_powers);
  interp_cd_integrated = zeros(n_rho,nb_launchers+1,nt_ec_powers);
  for it = 1:nt_ec_powers
    interp_p_dens(:,:,it)        = interp_norm_p_dens(:,:,it).*p_ec_injected(it,:);
    interp_p_integrated(:,:,it)  = interp_norm_p_integrated(:,:,it).*p_ec_injected(it,:);
    interp_cd_dens(:,:,it)       = interp_norm_cd_dens(:,:,it).*p_ec_injected(it,:);
    interp_cd_integrated(:,:,it) = interp_norm_cd_integrated(:,:,it).*p_ec_injected(it,:);
  end

  % fill profiles_1d with interpolated profiles
  for i_lau = active_launchers
    for ii = 1:nt_ec_powers  
      % time
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.time = ec_powers_tgrid(ii); 
      % rhotor grid
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.grid.rho_tor_norm = ...
        ec_data.p_dens.rhotor_norm(1,:); % constant in time
      % power density
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.electrons.energy = ...
        interp_p_dens(:,i_lau,ii);
      % integrated power density
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.electrons.power_inside = ...
        interp_p_integrated(:,i_lau,ii);
      % current density (to adapt to <J.B>/B0)
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.j_parallel = ...
        interp_cd_dens(:,i_lau,ii);
      % integrated current density (to adapt to INTEGRAL(<J.B>/B0)*ds_phi)
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.current_parallel_inside = ...
        interp_cd_integrated(:,i_lau,ii); 
    end
  end

  %add empty sources for rest of unsused launchers
  if numel(ids_core_sources.source)-last_index ~= nb_launchers
    ids_core_sources.source{last_index+nb_launchers} = [];
  end    

  last_index = last_index+nb_launchers;

end

%% nbi  
id_nbi.description = 'Source from Neutral Beam Injection';
id_nbi.index = 2; id_nbi.name = 'nbi';
%ids_core_sources.source{} = source_template;
%ids_core_sources.source{}.identifier = id_nbi;
%ids_core_sources.source{}.profiles_1d(1:n_t) = {profiles_template};
%ids_core_sources.source{}.global_quantities(1:n_t) = {globals_template};


% total  
id_total.description = 'Total source; combines all sources';
id_total.index = 1; id_total.name = 'total';
%ids_core_sources.source{} = source_template;
%ids_core_sources.source{}.identifier = id_total;
%ids_core_sources.source{}.profiles_1d(1:n_t) = {profiles_template};
%ids_core_sources.source{}.global_quantities(1:n_t) = {globals_template};


%% add descriptions for profiles_1d

desc.source = 'available by now {source_index}: ohmic {1}, bootstrap {2}, ec {3+} for individual launchers (if present in shot)';
desc.profiles_1d.electrons.energy = 'absorbed power density';
desc.profiles_1d.electrons.power_inside = 'integrated absorbed power density';
desc.profiles_1d.j_parallel = 'parallel current density';
desc.profiles_1d.current_parallel_inside = 'integrated parallel current density';
desc.globals.power = 'total power coupled to the plasma';
desc.globals.current_parallel = 'total parallel current driven';
ids_core_sources_description = desc;

%%
if nargin <= 2
  ids_core_sources.code.name = ['tcv_get_ids_core_sources, within gdat, with shot= ' num2str(params_core_sources.shot) ];
else
  ids_core_sources.code.name = ['tcv_get_ids_core_sources, within gdat, with shot= ' num2str(params_core_sources.shot) '; varargin: ' varargin{:}];
end
ids_core_sources_description.code.name = ids_core_sources.code.name;

ids_core_sources.code.output_flag = zeros(size(ids_core_sources.time));

% cocos automatic transform
if ~isempty(which('ids_generic_cocos_nodes_transformation_symbolic'))
  [ids_core_sources,cocoscoeff]=ids_generic_cocos_nodes_transformation_symbolic(ids_core_sources,'core_sources',gdat_params.cocos_in, ...
          gdat_params.cocos_out,gdat_params.ipsign_out,gdat_params.b0sign_out,gdat_params.ipsign_in,gdat_params.b0sign_in, ...
          gdat_params.error_bar,gdat_params.nverbose);

end